Re: [PATCH v3 13/18] of: overlay: check prevents multiple fragments touching same property

2018-10-15 Thread Frank Rowand
On 10/14/18 20:21, Frank Rowand wrote:
> On 10/14/18 18:55, Joe Perches wrote:
>> On Sun, 2018-10-14 at 18:52 -0700, Frank Rowand wrote:
>>> On 10/14/18 18:06, Joe Perches wrote:
 On Sun, 2018-10-14 at 17:24 -0700, frowand.l...@gmail.com wrote:
> From: Frank Rowand 
>
> Add test case of two fragments updating the same property.  After
> adding the test case, the system hangs at end of boot, after
> after slub stack dumps from kfree() in crypto modprobe code.
>> []
 I think this is worse performance than before.

 This now walks all entries when before it would
 return -EINVAL directly when it found a match.
>>>
>>> Yes, it is worse performance, but that is OK.
>>>
>>> This is a check that is done when a devicetree overlay is applied.
>>> If an error occurs then that means that the overlay was incorrectly
>>> specified.  The file drivers/of/unittest-data/overlay_bad_add_dup_prop.dts
>>> in this patch provides an example of how a bad overlay can be created.
>>>
>>> Once an error was detected, the check could return immediately, or it
>>> could continue to give a complete list of detected errors.  I chose to
>>> give the complete list of detected errors.
>>
>> Swell.  Please describe that in the commit message.
> 
> If a version 4 of the series is created I will update the commit
> message.  As a stand alone item I do not think it is worth a
> new version.

And there will be a version 4, so I will update the commit message.

-Frank


Re: [PATCH v3 13/18] of: overlay: check prevents multiple fragments touching same property

2018-10-14 Thread Frank Rowand
On 10/14/18 18:55, Joe Perches wrote:
> On Sun, 2018-10-14 at 18:52 -0700, Frank Rowand wrote:
>> On 10/14/18 18:06, Joe Perches wrote:
>>> On Sun, 2018-10-14 at 17:24 -0700, frowand.l...@gmail.com wrote:
 From: Frank Rowand 

 Add test case of two fragments updating the same property.  After
 adding the test case, the system hangs at end of boot, after
 after slub stack dumps from kfree() in crypto modprobe code.
> []
>>> I think this is worse performance than before.
>>>
>>> This now walks all entries when before it would
>>> return -EINVAL directly when it found a match.
>>
>> Yes, it is worse performance, but that is OK.
>>
>> This is a check that is done when a devicetree overlay is applied.
>> If an error occurs then that means that the overlay was incorrectly
>> specified.  The file drivers/of/unittest-data/overlay_bad_add_dup_prop.dts
>> in this patch provides an example of how a bad overlay can be created.
>>
>> Once an error was detected, the check could return immediately, or it
>> could continue to give a complete list of detected errors.  I chose to
>> give the complete list of detected errors.
> 
> Swell.  Please describe that in the commit message.

If a version 4 of the series is created I will update the commit
message.  As a stand alone item I do not think it is worth a
new version.

-Frank


Re: [PATCH v3 13/18] of: overlay: check prevents multiple fragments touching same property

2018-10-14 Thread Joe Perches
On Sun, 2018-10-14 at 18:52 -0700, Frank Rowand wrote:
> On 10/14/18 18:06, Joe Perches wrote:
> > On Sun, 2018-10-14 at 17:24 -0700, frowand.l...@gmail.com wrote:
> > > From: Frank Rowand 
> > > 
> > > Add test case of two fragments updating the same property.  After
> > > adding the test case, the system hangs at end of boot, after
> > > after slub stack dumps from kfree() in crypto modprobe code.
[]
> > I think this is worse performance than before.
> > 
> > This now walks all entries when before it would
> > return -EINVAL directly when it found a match.
> 
> Yes, it is worse performance, but that is OK.
> 
> This is a check that is done when a devicetree overlay is applied.
> If an error occurs then that means that the overlay was incorrectly
> specified.  The file drivers/of/unittest-data/overlay_bad_add_dup_prop.dts
> in this patch provides an example of how a bad overlay can be created.
> 
> Once an error was detected, the check could return immediately, or it
> could continue to give a complete list of detected errors.  I chose to
> give the complete list of detected errors.

Swell.  Please describe that in the commit message.




Re: [PATCH v3 13/18] of: overlay: check prevents multiple fragments touching same property

2018-10-14 Thread Frank Rowand
On 10/14/18 18:06, Joe Perches wrote:
> On Sun, 2018-10-14 at 17:24 -0700, frowand.l...@gmail.com wrote:
>> From: Frank Rowand 
>>
>> Add test case of two fragments updating the same property.  After
>> adding the test case, the system hangs at end of boot, after
>> after slub stack dumps from kfree() in crypto modprobe code.
> []
>> -static int check_changeset_dup_add_node(struct overlay_changeset *ovcs)
>> +static int changeset_dup_entry_check(struct overlay_changeset *ovcs)
>>  {
>> -struct of_changeset_entry *ce_1, *ce_2;
>> -char *fn_1, *fn_2;
>> -int name_match;
>> +struct of_changeset_entry *ce_1;
>> +int dup_entry = 0;
>>  
>>  list_for_each_entry(ce_1, &ovcs->cset.entries, node) {
>> -
>> -if (ce_1->action == OF_RECONFIG_ATTACH_NODE ||
>> -ce_1->action == OF_RECONFIG_DETACH_NODE) {
>> -
>> -ce_2 = ce_1;
>> -list_for_each_entry_continue(ce_2, &ovcs->cset.entries, 
>> node) {
>> -if (ce_2->action == OF_RECONFIG_ATTACH_NODE ||
>> -ce_2->action == OF_RECONFIG_DETACH_NODE) {
>> -/* inexpensive name compare */
>> -if (!of_node_cmp(ce_1->np->full_name,
>> -ce_2->np->full_name)) {
>> -/* expensive full path name 
>> compare */
>> -fn_1 = kasprintf(GFP_KERNEL, 
>> "%pOF", ce_1->np);
>> -fn_2 = kasprintf(GFP_KERNEL, 
>> "%pOF", ce_2->np);
>> -name_match = !strcmp(fn_1, 
>> fn_2);
>> -kfree(fn_1);
>> -kfree(fn_2);
>> -if (name_match) {
>> -pr_err("ERROR: multiple 
>> overlay fragments add and/or delete node %pOF\n",
>> -   ce_1->np);
>> -return -EINVAL;
>> -}
>> -}
>> -}
>> -}
>> -}
>> +dup_entry |= find_dup_cset_node_entry(ovcs, ce_1);
>> +dup_entry |= find_dup_cset_prop(ovcs, ce_1);
> 
> I think this is worse performance than before.
> 
> This now walks all entries when before it would
> return -EINVAL directly when it found a match.

Yes, it is worse performance, but that is OK.

This is a check that is done when a devicetree overlay is applied.
If an error occurs then that means that the overlay was incorrectly
specified.  The file drivers/of/unittest-data/overlay_bad_add_dup_prop.dts
in this patch provides an example of how a bad overlay can be created.

Once an error was detected, the check could return immediately, or it
could continue to give a complete list of detected errors.  I chose to
give the complete list of detected errors.

-Frank


Re: [PATCH v3 13/18] of: overlay: check prevents multiple fragments touching same property

2018-10-14 Thread Joe Perches
On Sun, 2018-10-14 at 17:24 -0700, frowand.l...@gmail.com wrote:
> From: Frank Rowand 
> 
> Add test case of two fragments updating the same property.  After
> adding the test case, the system hangs at end of boot, after
> after slub stack dumps from kfree() in crypto modprobe code.
[]
> -static int check_changeset_dup_add_node(struct overlay_changeset *ovcs)
> +static int changeset_dup_entry_check(struct overlay_changeset *ovcs)
>  {
> - struct of_changeset_entry *ce_1, *ce_2;
> - char *fn_1, *fn_2;
> - int name_match;
> + struct of_changeset_entry *ce_1;
> + int dup_entry = 0;
>  
>   list_for_each_entry(ce_1, &ovcs->cset.entries, node) {
> -
> - if (ce_1->action == OF_RECONFIG_ATTACH_NODE ||
> - ce_1->action == OF_RECONFIG_DETACH_NODE) {
> -
> - ce_2 = ce_1;
> - list_for_each_entry_continue(ce_2, &ovcs->cset.entries, 
> node) {
> - if (ce_2->action == OF_RECONFIG_ATTACH_NODE ||
> - ce_2->action == OF_RECONFIG_DETACH_NODE) {
> - /* inexpensive name compare */
> - if (!of_node_cmp(ce_1->np->full_name,
> - ce_2->np->full_name)) {
> - /* expensive full path name 
> compare */
> - fn_1 = kasprintf(GFP_KERNEL, 
> "%pOF", ce_1->np);
> - fn_2 = kasprintf(GFP_KERNEL, 
> "%pOF", ce_2->np);
> - name_match = !strcmp(fn_1, 
> fn_2);
> - kfree(fn_1);
> - kfree(fn_2);
> - if (name_match) {
> - pr_err("ERROR: multiple 
> overlay fragments add and/or delete node %pOF\n",
> -ce_1->np);
> - return -EINVAL;
> - }
> - }
> - }
> - }
> - }
> + dup_entry |= find_dup_cset_node_entry(ovcs, ce_1);
> + dup_entry |= find_dup_cset_prop(ovcs, ce_1);

I think this is worse performance than before.

This now walks all entries when before it would
return -EINVAL directly when it found a match.




[PATCH v3 13/18] of: overlay: check prevents multiple fragments touching same property

2018-10-14 Thread frowand . list
From: Frank Rowand 

Add test case of two fragments updating the same property.  After
adding the test case, the system hangs at end of boot, after
after slub stack dumps from kfree() in crypto modprobe code.

Multiple overlay fragments adding, modifying, or deleting the same
property is not supported.  Add check to detect the attempt and fail
the overlay apply.

After applying this patch, the devicetree unittest messages will
include:

   OF: overlay: ERROR: multiple overlay fragments add, update, and/or delete 
property /testcase-data-2/substation@100/motor-1/rpm_avail

   ...

   ### dt-test ### end of unittest - 212 passed, 0 failed

The check to detect two fragments updating the same property is
folded into the patch that created the test case to maintain
bisectability.

Signed-off-by: Frank Rowand 
---

Changes since v2:
  - Use continue to reduce indentation in find_dup_cset_node_entry()
and find_dup_cset_prop()

 drivers/of/overlay.c   | 118 ++---
 drivers/of/unittest-data/Makefile  |   1 +
 .../of/unittest-data/overlay_bad_add_dup_prop.dts  |  24 +
 drivers/of/unittest-data/overlay_base.dts  |   1 +
 drivers/of/unittest.c  |   5 +
 5 files changed, 112 insertions(+), 37 deletions(-)
 create mode 100644 drivers/of/unittest-data/overlay_bad_add_dup_prop.dts

diff --git a/drivers/of/overlay.c b/drivers/of/overlay.c
index b0a0dafb6a13..908bcc62019b 100644
--- a/drivers/of/overlay.c
+++ b/drivers/of/overlay.c
@@ -514,52 +514,96 @@ static int build_changeset_symbols_node(struct 
overlay_changeset *ovcs,
return 0;
 }
 
+static int find_dup_cset_node_entry(struct overlay_changeset *ovcs,
+   struct of_changeset_entry *ce_1)
+{
+   struct of_changeset_entry *ce_2;
+   char *fn_1, *fn_2;
+   int node_path_match;
+
+   if (ce_1->action != OF_RECONFIG_ATTACH_NODE &&
+   ce_1->action != OF_RECONFIG_DETACH_NODE)
+   return 0;
+
+   ce_2 = ce_1;
+   list_for_each_entry_continue(ce_2, &ovcs->cset.entries, node) {
+   if ((ce_2->action != OF_RECONFIG_ATTACH_NODE &&
+ce_2->action != OF_RECONFIG_DETACH_NODE) ||
+   of_node_cmp(ce_1->np->full_name, ce_2->np->full_name))
+   continue;
+
+   fn_1 = kasprintf(GFP_KERNEL, "%pOF", ce_1->np);
+   fn_2 = kasprintf(GFP_KERNEL, "%pOF", ce_2->np);
+   node_path_match = !strcmp(fn_1, fn_2);
+   kfree(fn_1);
+   kfree(fn_2);
+   if (node_path_match) {
+   pr_err("ERROR: multiple overlay fragments add and/or 
delete node %pOF\n",
+  ce_1->np);
+   return -EINVAL;
+   }
+   }
+
+   return 0;
+}
+
+static int find_dup_cset_prop(struct overlay_changeset *ovcs,
+   struct of_changeset_entry *ce_1)
+{
+   struct of_changeset_entry *ce_2;
+   char *fn_1, *fn_2;
+   int node_path_match;
+
+   if (ce_1->action != OF_RECONFIG_ADD_PROPERTY &&
+   ce_1->action != OF_RECONFIG_REMOVE_PROPERTY &&
+   ce_1->action != OF_RECONFIG_UPDATE_PROPERTY)
+   return 0;
+
+   ce_2 = ce_1;
+   list_for_each_entry_continue(ce_2, &ovcs->cset.entries, node) {
+   if ((ce_2->action != OF_RECONFIG_ADD_PROPERTY &&
+ce_2->action != OF_RECONFIG_REMOVE_PROPERTY &&
+ce_2->action != OF_RECONFIG_UPDATE_PROPERTY) ||
+   of_node_cmp(ce_1->np->full_name, ce_2->np->full_name))
+   continue;
+
+   fn_1 = kasprintf(GFP_KERNEL, "%pOF", ce_1->np);
+   fn_2 = kasprintf(GFP_KERNEL, "%pOF", ce_2->np);
+   node_path_match = !strcmp(fn_1, fn_2);
+   kfree(fn_1);
+   kfree(fn_2);
+   if (node_path_match &&
+   !of_prop_cmp(ce_1->prop->name, ce_2->prop->name)) {
+   pr_err("ERROR: multiple overlay fragments add, update, 
and/or delete property %pOF/%s\n",
+  ce_1->np, ce_1->prop->name);
+   return -EINVAL;
+   }
+   }
+
+   return 0;
+}
+
 /**
  * check_changeset_dup_add_node() - changeset validation: duplicate add node
  * @ovcs:  Overlay changeset
  *
  * Check changeset @ovcs->cset for multiple add node entries for the same
  * node.
  *
- * Returns 0 on success, -ENOMEM if memory allocation failure, or -EINVAL if
- * invalid overlay in @ovcs->fragments[].
+ * Returns 0 on success, or -EINVAL if duplicate changeset entry found.
  */
-static int check_changeset_dup_add_node(struc
@@ -514,52 +514,96 @@ static int build_changeset_symbols_node(struct 
 /**
- * check_changeset_dup_add_node() - changeset validation: duplicate add node
+ * changeset_dup_entry_check() - check for duplicate entries
  * @ovcs:  Overlay changeset
  *
@@ -514,52 +514,96 @@ static int build_changeset_symbols_node(struct 
  * @ovcs:  Overlay changeset
  *
- * Check changeset @ovcs->cset for multiple add node entries for the same
- * node.
+ * Check changeset @ovcs->cset for multiple {add or delete} node entries for
+ * the same node or duplicate {add, delete, or update} properties entries
+ * for the same property.
  *